#include <stdint.h>

#include <pcf8833.h>
#include <sdram.h>
#include <spi.h>
#include <utils.h>

#define DMA_LENGTH		(72*1024)

static uint16_t *dma = (uint16_t*) (SDRAM_BASE_ADDRESS + (SDRAM_SIZE - DMA_LENGTH));
static int pcf8833_cs;

void static
pcf8833_transfer(const uint8_t *ptr, int len)
{
	struct spi_tblk blk;
	int i;

	for (i = 0; i < len; ++i)
		*(dma + i) = 0x0100 | *(ptr + i);

	blk.rx = dma;
	blk.tx = dma;
	blk.len = len;

	spi_select(pcf8833_cs);
	spi_transfer(&blk, 1);
}

void static
pcf8833_command(uint8_t cmd)
{
	struct spi_tblk blk;
	uint16_t data = (uint16_t) cmd;

	blk.rx = &data;
	blk.tx = &data;
	blk.len = 1;

	spi_select(pcf8833_cs);
	spi_transfer(&blk, 1);
}

void static
pcf8833_data(uint8_t val)
{
	struct spi_tblk blk;
	uint16_t data = 0x0100 | (uint16_t) val;

	blk.rx = &data;
	blk.tx = &data;
	blk.len = 1;

	spi_select(pcf8833_cs);
	spi_transfer(&blk, 1);
}

int pcf8833_init(int cs, int flags)
{
	pcf8833_cs = cs;

	spi_mode(pcf8833_cs, SPI_MODE_0, 9, 4, 0);

	pcf8833_command(0x01);
	nops(100);
	pcf8833_command(0x11);
	nops(100);
	pcf8833_command(0x03);
	nops(100);
	pcf8833_command(0x29);
	nops(100);
	pcf8833_command(0x20);
	nops(100);

	pcf8833_command(0x36);
	pcf8833_data(flags & 0x000000ff);

	pcf8833_command(0x3a);
	pcf8833_data((flags >> 8) & 0x000000ff);

	return 0;
}

int pcf8833_pwoff()
{
	pcf8833_command(0x28);

	return 0;
}

int pcf8833_draw(	int xs,
			int ys,
			int xe,
			int ye,
			const void *ptr)
{
	int len = (xe - xs + 1) * (ye - ys + 1);

	pcf8833_command(0x2a);
	pcf8833_data((uint8_t) xs);
	pcf8833_data((uint8_t) xe);

	pcf8833_command(0x2b);
	pcf8833_data((uint8_t) ys);
	pcf8833_data((uint8_t) ye);

	pcf8833_command(0x2c);
	pcf8833_transfer(ptr, len);

	return 0;
}

