#include <stdint.h>

#include <AT91RM9200.h>
#include <config.h>
#include <sdram.h>
#include <serial.h>
#include <spi.h>
#include <utils.h>

/* for 16.0 MHz crystal
 * */
#define CLOCK_PLLAR_VAL		0x202cbf04
#define CLOCK_PLLBR_VAL		0x10173f04

static void
wait_lock(int bit)
{
	int i = 0;
	while (!(AT91C_BASE_PMC->PMC_SR & bit) && (i++ < 128))
		nops(16);
}

static void
z_init_clock()
{
	AT91C_BASE_PMC->PMC_IDR = 0xffffffff;

	AT91C_BASE_CKGR->CKGR_MOR = AT91C_CKGR_MOSCEN | (0xff << 8);
	wait_lock(AT91C_PMC_MOSCS);

	AT91C_BASE_CKGR->CKGR_PLLAR = CLOCK_PLLAR_VAL;
	wait_lock(AT91C_PMC_LOCKA);

	AT91C_BASE_CKGR->CKGR_PLLBR = CLOCK_PLLBR_VAL;
	wait_lock(AT91C_PMC_LOCKB);

	AT91C_BASE_PMC->PMC_MCKR = (AT91C_PMC_MDIV_2 | AT91C_PMC_CSS_PLLA_CLK);
	wait_lock(AT91C_PMC_MCKRDY);

	AT91C_BASE_PMC->PMC_SCER = AT91C_PMC_PCK;
}

void zboot_init()
{
	z_init_clock();

	sdram_init();
	serial_init();
	spi_init();

	nops(1000);
}

